import { ec } from 'elliptic';
import { SHA256, AES, enc } from 'crypto-js';

export class Cryptrack {
    private readonly _curve: ec;
    private readonly _keys: ec.KeyPair;
    private readonly _store: Map<string, string>;

    constructor() {
        this._curve = new ec('secp256k1');
        this._keys = this._curve.genKeyPair();
        this._store = new Map();
    }

    /**
     * Регистрирует сервис, вычисляет sharedKey и сохраняет его в памяти
     */
    register(id: string, publicKeyHex: string): string {
        try {
            const peerPubKey = this._curve.keyFromPublic(publicKeyHex, 'hex').getPublic();
            const sharedSecret = this._keys.derive(peerPubKey).toString(16);
            const sharedKey = SHA256(sharedSecret).toString();

            this._store.set(id, sharedKey);
            return sharedKey;
        } catch (err) {
            throw new Error(`Invalid public key: ${(err as Error).message}`);
        }
    }

    /**
     * Возвращает sharedKey по id (если был ранее зарегистрирован)
     */
    getKey(id: string): string | undefined {
        return this._store.get(id);
    }

    /**
     * Шифрует строку с использованием sharedKey по id
     */
    encrypt(id: string, data: string): string {
        const key = this.getKey(id);
        if (!key) throw new Error(`Shared key for ID "${id}" not found`);

        return AES.encrypt(data, key).toString();
    }

    /**
     * Расшифровывает строку с использованием sharedKey по id
     */
    decrypt(id: string, encrypted: string): string {
        const key = this.getKey(id);
        if (!key) throw new Error(`Shared key for ID "${id}" not found`);

        try {
            const bytes = AES.decrypt(encrypted, key);
            const decrypted = bytes.toString(enc.Utf8);
            if (!decrypted) throw new Error('Failed to decode decrypted data');
            return decrypted;
        } catch (err) {
            throw new Error(`Decryption failed: ${(err as Error).message}`);
        }
    }

    /**
     * Возвращает публичный ключ текущего Bilibo
     */
    getPublicKey(): string {
        return this._keys.getPublic('hex');
    }
}
